#version 330

uniform sampler2D texDiffuseColor;
uniform sampler2D texFragPosition;
uniform sampler2D texFragNormal;

uniform vec3 lightPositionA;
uniform vec3 lightPositionB;
uniform vec3 ambientColor;
uniform vec3 diffuseColor;
uniform vec3 specularColor;
uniform vec3 attenuation;

uniform vec3 eyePosition;

smooth in vec2 fragmentUV;

out vec4 fragColor;

vec3 closestPointOnLine(vec3 point, vec3 lineA, vec3 lineB)
{
	vec3 lineDir = lineB - lineA;
	float lineLength = length(lineDir);
	lineDir = normalize(lineDir);

	vec3 v = point - lineA;

	// Project v to LineDirection to get the distance of point from a
	float dist = dot(v, lineDir);
	
	if(dist <= 0) {
		return lineA;
	}
	else {
		if(dist >= lineLength)  {
			return lineB;
		}
		else {
			return lineA + lineDir * dist;
		}
	}
}

void main(void)
{
	vec3 lightPosA = vec3(vec4(lightPositionA, 1.0));
	vec3 lightPosB = vec3(vec4(lightPositionB, 1.0));

	float fragLightDist;
	vec3 fragLightDir;
	float luminocity;
	float diffuseCoefficient;
	vec3 reflection;
	float specular;
	vec3 fragPosition = texture(texFragPosition, fragmentUV).xyz;
	vec3 fragNormal = texture(texFragNormal, fragmentUV).xyz;
	vec4 fragDiffuse = texture(texDiffuseColor, fragmentUV);

	fragColor = vec4(0.0, 0.0, 0.0, 1.0);

	// calculate closest point
	vec3 lightPos = closestPointOnLine(fragPosition, lightPosA, lightPosB);

	fragLightDist = distance(lightPos.xyz, fragPosition);
	fragLightDir = normalize(lightPos.xyz - fragPosition);

	luminocity = 1.0f / (attenuation.x + attenuation.y * fragLightDist
			+ attenuation.z * fragLightDist * fragLightDist);

	// add ambient color of this light source
	fragColor.rgb += ambientColor.rgb;
	
	// dot product of normal and lightdir for phong diffuse light
	diffuseCoefficient = max(0.0, dot(fragNormal, fragLightDir));
	
	// add diffuse light to ambient light
	fragColor.rgb += diffuseCoefficient * diffuseColor.rgb;

	fragColor.rgb *= luminocity * fragDiffuse.rgb;
	fragColor.a = fragDiffuse.a;

	vec3 viewDirection = normalize(eyePosition - fragPosition);

	specular = max(0.0, dot(reflect(-fragLightDir, fragNormal), viewDirection));

	if (diffuseCoefficient != 0)
	{
		specular = pow(specular, 128.0) * luminocity;
		fragColor.rgb += specularColor * specular;
	}
}
