#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 50) out;

in float type0[];
in vec3 position0[];
in vec3 velocity0[];
in float age0[];
in float alpha0[];

out float type1;
out vec3 position1;
out vec3 velocity1;
out float age1;
out float alpha1;

uniform float time;
uniform float time_delta;
uniform sampler2D randomTexture;

#define PARTICLE_TYPE_LAUNCHER 0.0f
#define PARTICLE_TYPE_SHELL 1.0f
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f

#define LAUNCHER_LIFETIME 0.0f
#define SHELL_LIFETIME 0.15f
#define SECONDARY_SHELL_LIFETIME 1.5f

vec3 randomDirection(float u)                    
{
	 vec3 dir = texture(randomTexture, vec2(u, 0.f)).xyz;
	 dir -= vec3(0.5, 0.5, 0.5);
	 return dir;
}

void main()
{
	float age = age0[0] + time_delta;

	if (type0[0] == PARTICLE_TYPE_LAUNCHER) {
		if (age >= LAUNCHER_LIFETIME) {
			for(int i = 0 ; i < 40 ; i++) {
				type1 = PARTICLE_TYPE_SHELL;
				position1 = position0[0];
				vec3 dir = randomDirection(time+i/40.f);
				velocity1 = normalize(dir)*4.f;
				velocity1.y *= 2.f;
				age1 = 0.0;
				alpha1 = 1.0;
				EmitVertex();
				EndPrimitive();
			}
			age = 0.0;
		}
		else {
			// respawn launcher
			type1 = PARTICLE_TYPE_LAUNCHER;
			position1 = position0[0];
			velocity1 = velocity0[0];
			age1 = age;
			alpha1 = alpha0[0];
			EmitVertex();
			EndPrimitive();
		}
	}
	else {
		vec3 deltaP = time_delta * velocity0[0];
		vec3 pull = time_delta * vec3(0.f, -0.5f, 0.f);

		if(type0[0] == PARTICLE_TYPE_SHELL) {
			if(age < SHELL_LIFETIME) {
				type1 = PARTICLE_TYPE_SHELL;
				position1 = position0[0] + deltaP;
				velocity1 = velocity0[0] + pull;
				age1 = age;
				alpha1 = alpha0[0];
				EmitVertex();
				EndPrimitive();
			}
			else {
				for(int i = 0 ; i < 10 ; i++) {
					type1 = PARTICLE_TYPE_SECONDARY_SHELL;
					position1 = position0[0];
					vec3 dir = randomDirection(time+i/10.f);
					dir.y = max(dir.y, 0.5);
					velocity1 = normalize(dir)*1.3f;
					age1 = 0.0f;
					alpha1 = alpha0[0];
					EmitVertex();
					EndPrimitive();
				}
			}
		}
		else {
			// type = secondy shell
			if(age < SECONDARY_SHELL_LIFETIME) {
				type1 = PARTICLE_TYPE_SECONDARY_SHELL;
				position1 = position0[0] + deltaP;
				velocity1 = velocity0[0] + pull;
				age1 = age;
				alpha1 = 1.f - age/SECONDARY_SHELL_LIFETIME;
				EmitVertex();
				EndPrimitive();
			}
		}
	}
}